package xmipp.viewer.particlepicker.training.gui;

import xmipp.jni.Classifier;
import xmipp.jni.Classifier.Parameter;
import xmipp.utils.*;
import xmipp.viewer.ctf.CTFAnalyzerJFrame;
import xmipp.viewer.particlepicker.*;
import xmipp.viewer.particlepicker.training.model.*;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.TableColumnModel;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class SupervisedPickerJFrame extends ParticlePickerJFrame {

    public static final Color THRESHOLD_LABEL_CHANGE_COLOR = Color.DARK_GRAY;
    protected SupervisedPickerCanvas canvas;
    protected JMenuBar mb;
    protected SupervisedParticlePicker ppicker;
    protected JPanel micrographpn;
    protected MicrographsTableModel micrographsmd;

    protected float positionx;
    protected JButton iconbt;
    protected JLabel manuallb;
    protected JLabel autolb;
    protected JLabel savedlb;

    protected JSlider thresholdsl;
    protected JPanel thresholdpn;
    protected JFormattedTextField thresholdtf;

    protected JToggleButton centerparticlebt;
    protected JMenuItem exportmi;
    protected JToggleButton autopickchb;
    protected JPanel sppickerpn;
    
    protected JMenuItem templatesmi;
    TemplatesJDialog templatesdialog;
    protected JLabel checkpercentlb;
    protected JFormattedTextField autopickpercenttf;
    protected JLabel thresholdlb;
    protected JPanel gpickerpn;
	private JButton autopickbt;
	private HashMap<JTextField, Parameter> paramtfs;

    @Override
    public SupervisedParticlePicker getParticlePicker() {
        return ppicker;
    }

    public SupervisedPickerJFrame(SupervisedParticlePicker picker) {

        super(picker);
        try {
            this.ppicker = picker;
            initComponents();
            setChanged(false);
            Classifier c = ppicker.getClassifier();
            if(c instanceof GenericClassifier) {
                GenericClassifier gc = (GenericClassifier)c;
                if (!gc.hasInitialCoordinates())
                    ppicker.autopick(this, getMicrograph());
            }
        } catch (IllegalArgumentException ex) {
            //close();
            throw ex;
        }
    }

    public boolean isCenterParticle() {
        return centerparticlebt.isSelected();
    }

    public boolean centerParticle() {
        return centerparticlebt.isSelected() && isNormalMode();
    }

    @Override
    public ParticlesDialog initParticlesJDialog() {
        return new ParticlesDialog(this);
    }

    public SupervisedPickerMicrograph getMicrograph() {
        return ppicker.getMicrograph();
    }

    public double getThreshold() {
        if (thresholdsl == null) {
            return 0;
        }
        return thresholdsl.getValue() / 100.0;
    }

    @Override
    public List<? extends ManualParticle> getAvailableParticles() {
        return getMicrograph().getAvailableParticles(getThreshold());
    }

    public String importParticlesFromFile(Format format, String file, float scale, boolean invertx, boolean inverty) {
        if (!new File(file).exists()) {
            throw new IllegalArgumentException(XmippMessage.getNoSuchFieldValueMsg("file", file));
        }
        String result = "";
        if (ppicker.isReviewFile(file)) {
            result = ppicker.importAllParticles(file, scale, invertx, inverty);
            ppicker.saveAllData();
        } else {
            result = importMicrographParticles(format, file, scale, invertx, inverty);
        }
        setChanged(false);
        getCanvas().repaint();
        updateMicrographsModel();
        updateSize(ppicker.getSize());//will also update templates
        canvas.refreshActive(null);

        return result;
    }

    public String importMicrographParticles(Format format, String file, float scale, boolean invertx, boolean inverty) {

        String filename = Micrograph.getName(file, 1);
		// validating you want use this file for this micrograph with different
        // name
        if (!filename.equals(getMicrograph().getName())) {
            String msg = String.format("Are you sure you want to import data from file\n%s to micrograph %s ?", file, getMicrograph().getName());
            boolean importdata = XmippDialog.showQuestion(this, msg);
            if (!importdata) {
                return null;
            }
        }
        ppicker.resetMicrograph(getMicrograph());
        String result = ppicker.importParticlesFromFile(file, format, getMicrograph(), scale, invertx, inverty);
        ppicker.saveData(getMicrograph());
        return result;
    }

    public void updateSize(int size) {
        try {
            
            super.updateSize(size);
            ppicker.updateTemplatesStack(true);
            

        } catch (Exception e) {
        	e.printStackTrace();
            XmippDialog.showError(this, e.getMessage());
        }
    }

    @Override
    public String importParticles(Format format, String dir, String preffix, String suffix, float scale, boolean invertx, boolean inverty) {
        String result = "";

        if (new File(dir).isDirectory()) {
            result = ppicker.importParticlesFromFolder(dir, format, preffix, suffix, scale, invertx, inverty);
            boolean resize = ((Integer)sizetf.getValue()).intValue() != ppicker.getSize();
            sizetf.setValue(ppicker.getSize());
            getCanvas().repaint();
            updateMicrographsModel(true);
            getCanvas().refreshActive(null);
            ppicker.updateTemplatesStack(resize);
            

        } else // only can choose file if TrainingPickerJFrame instance
        {
            result = importParticlesFromFile(format, dir, scale, invertx, inverty);
        }
        return result;

    }


    protected void initializeCanvas() {

        if (canvas == null) {
            canvas = new SupervisedPickerCanvas(this);
        } else {
            canvas.updateMicrograph();
        }

    }

    protected void formatMicrographsTable() {
        int totalWidth = 630;
        int micWidth = 440;

        if (ppicker.containsPSD()) // shrink columns to make space for PSD
        {
            totalWidth -= 115;
            micWidth -= 115;
        }

        micrographstb.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

        TableColumnModel tcm = micrographstb.getColumnModel();
        tcm.getColumn(0).setPreferredWidth(40);

        if (ppicker.hasDefocusU())
        {
            micWidth -= 70;
            tcm.getColumn(4).setPreferredWidth(70);
        }
        
        tcm.getColumn(1).setPreferredWidth(micWidth);
        tcm.getColumn(2).setPreferredWidth(70);
        tcm.getColumn(3).setPreferredWidth(80);
        micrographstb.setPreferredScrollableViewportSize(new Dimension(totalWidth, 304));
        micrographstb.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        int index = ppicker.getMicrographIndex();
        if (index != -1) {
            micrographstb.setRowSelectionInterval(index, index);
        }
    }

    void updateColor() {
        color = ppicker.getColor();
        colorbt.setIcon(new ColorIcon(color));
        canvas.repaint();
        ppicker.saveConfig();
    }
    public void setChanged(boolean changed) {
        ppicker.setChanged(changed);
        savemi.setEnabled(changed);
        savebt.setEnabled(changed);
    }

    public void setThresholdChanged(boolean changed){

        if (changed){
            this.thresholdlb.setForeground(THRESHOLD_LABEL_CHANGE_COLOR);
            this.setChanged(true);
        } else {
            this.thresholdlb.setForeground(Color.BLACK);
        }
    }
    public boolean hasThresholdChanged(){
        return this.thresholdlb.getForeground() == THRESHOLD_LABEL_CHANGE_COLOR;
    }

    public void setSaved(boolean saved) {
        ppicker.setSaved(saved);
        // Do not print anything: for debugging --> savedlb.setText(saved?"Saved":"");
    }

    public void updateMicrographsModel(boolean all) {

        if (particlesdialog != null)
            loadParticles(false);
        int index = ppicker.getMicrographIndex();

        if (all)
            micrographsmd.fireTableRowsUpdated(0, micrographsmd.getRowCount() - 1);
         else
            micrographsmd.fireTableRowsUpdated(index, index);

        micrographstb.setRowSelectionInterval(index, index);
        manuallb.setText(Integer.toString(ppicker.getManualParticlesNumber()));
        autolb.setText(Integer.toString(ppicker.getAutomaticParticlesNumber()));
    }

    public ParticlePickerCanvas getCanvas() {
        return canvas;
    }

    private void initComponents() {
        try {

            setResizable(false);
            setTitle();
            initMenuBar();
            setJMenuBar(mb);

            GridBagConstraints constraints = new GridBagConstraints();
            constraints.insets = new Insets(0, 5, 0, 5);
            constraints.anchor = GridBagConstraints.WEST;
            setLayout(new GridBagLayout());

            centerparticlebt = new JToggleButton(bundle.getString("center"), XmippResource.getIcon("center.png"));
            initToolBar();
            centerparticlebt.setSelected(true);
            tb.add(centerparticlebt);
            add(tb, XmippWindowUtil.getConstraints(constraints, 0, 0, 2, 1, GridBagConstraints.WEST));

            //add(shapepn, XmippWindowUtil.getConstraints(constraints, 1, 1));

            
            initSupervisedPickerPane();
            if(!ppicker.getClassifier().needsTraining())
            {
            	sppickerpn.setVisible(false);
            	initGenericPickerPane();
            	add(gpickerpn, XmippWindowUtil.getConstraints(constraints, 0, 3, 2, 1, GridBagConstraints.HORIZONTAL));
            }
            else
            	add(sppickerpn, XmippWindowUtil.getConstraints(constraints, 0, 3, 1, 1, GridBagConstraints.HORIZONTAL));
            add(sppickerpn, XmippWindowUtil.getConstraints(constraints, 1, 3, 1, 1, GridBagConstraints.HORIZONTAL));
            enableSupervised(ppicker.getMode() == Mode.Supervised);
            initMicrographsPane();
            add(micrographpn, XmippWindowUtil.getConstraints(constraints, 0, 4, 2, 1, GridBagConstraints.HORIZONTAL));
            JPanel actionspn = new JPanel(new FlowLayout(FlowLayout.RIGHT));
            actionspn.add(closebt);
            actionspn.add(savebt);
            actionspn.add(saveandexitbt);
            add(actionspn, XmippWindowUtil.getConstraints(constraints, 0, 5, 2, 1, GridBagConstraints.HORIZONTAL));
            if (ppicker.getMode() == Mode.ReadOnly) {
                enableEdition(false);
            }
            pack();
            positionx = 0.9f;
            XmippWindowUtil.setLocation(positionx, 0.2f, this);
            setVisible(true);
        } catch (Exception e) {
            e.printStackTrace();
            throw new IllegalArgumentException(e.getMessage());
        }
    }

    void loadTemplates() {

        try {
            if (templatesdialog == null) {
                templatesdialog = new TemplatesJDialog(this);
                ppicker.setTemplatesDialog(templatesdialog);
                ParticleToTemplatesTask.setTemplatesDialog(templatesdialog);
            } else {
                templatesdialog.setVisible(true);
            }
        } catch (Exception e) {
            XmippDialog.showError(this, e.getMessage());
        }

    }

    private void initSupervisedPickerPane() {
    	
        sppickerpn = new JPanel(new FlowLayout(FlowLayout.LEFT));
        sppickerpn.setBorder(BorderFactory.createTitledBorder(bundle.getString("autopick")));
        autopickchb = new JToggleButton("Activate Training", XmippResource.getIcon("pick.png"));
        autopickchb.setSelected(ppicker.isAutopick());
        autopickchb.setText(ppicker.isAutopick()? "Deactivate Training": "Activate Training");
        autopickchb.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                try {
                    boolean isautopick = autopickchb.isSelected();
                    
                    SupervisedPickerMicrograph trainmic = null;
                    if (isautopick) {
                    	trainmic = getTrainMic();
                        isautopick = trainmic != null;
                    }
                    if (isautopick) {
                        ppicker.setMode(Mode.Supervised);
                        ppicker.trainAndAutopick(SupervisedPickerJFrame.this, trainmic);
                        setTitle();

                    } else if (autopickchb.isSelected()) {
                        autopickchb.setSelected(false);
                    } else {
                        boolean ismanual = XmippDialog
                                .showQuestion(SupervisedPickerJFrame.this, "After this operation automatic particles will be converted to manual and classifier training lost. Are you sure you want to continue? ");
                        if (ismanual) {
                            ppicker.setMode(Mode.Manual);
                            resetbt.setEnabled(true);
                            ppicker.saveAllData();
                            updateMicrographsModel(true);
                            if (autopickchb.isSelected()) {
                                autopickchb.setSelected(false);
                            }
                            getMicrograph().resetParticlesRectangle();
                            canvas.refreshActive(null);
                            setTitle();
                        } else {
                            autopickchb.setSelected(true);
                        }
                    }

                    ppicker.saveConfig();
                    enableSupervised(autopickchb.isSelected());
                    autopickchb.setText(autopickchb.isSelected()? "Deactivate Training": "Activate Training");

                } catch (Exception ex) {
                    ex.printStackTrace();
                    XmippDialog.showError(SupervisedPickerJFrame.this, ex.getMessage());
                    autopickchb.setSelected(false);
                    return;
                }
            }
        });
        sppickerpn.add(autopickchb);
        initThresholdPane();
        sppickerpn.add(thresholdpn);
        checkpercentlb = new JLabel("Explore (%):");
        sppickerpn.add(checkpercentlb);
        autopickpercenttf = new JFormattedTextField(NumberFormat.getIntegerInstance());
        autopickpercenttf.setColumns(3);
        autopickpercenttf.setValue(ppicker.getAutopickpercent());
        autopickpercenttf.setEnabled(ppicker.getMode() == Mode.Supervised || ppicker.getMode() == Mode.Manual);
        autopickpercenttf.addActionListener(new ActionListener()
        {

                @Override
                public void actionPerformed(ActionEvent arg0)
                {

                        setAutopickPercent();

                }
        });

        sppickerpn.add(autopickpercenttf);

    }

    protected void setAutopickPercent()
    {
            if (autopickpercenttf.getValue() == null || ((Number)autopickpercenttf.getValue()).intValue() <= 0)
            {
                    XmippDialog.showInfo(this, XmippMessage.getEmptyFieldMsg("Check (%)"));
                    autopickpercenttf.setValue(getMicrograph().getAutopickpercent());
                    return;
            }

            int autopickpercent = ((Number) autopickpercenttf.getValue()).intValue();
            getMicrograph().setAutopickpercent(autopickpercent);
            ppicker.setAutopickpercent(autopickpercent);
            ppicker.saveConfig();
    }

    protected void enableSupervised(boolean selected) {

        thresholdsl.setEnabled(selected);
        thresholdlb.setEnabled(selected);
        thresholdtf.setEnabled(selected);
        sizelb.setEnabled(!selected);
        // in sup mode size cannot be changed
        sizetf.setEnabled(!selected);
        sizelb.setEnabled(!selected);
        importmi.setEnabled(!selected);
        //autopickpercenttf.setEnabled(selected);
        
        pack();

    }

    public boolean isSupervised() {
        return autopickchb.isSelected();
    }

    protected void enableEdition(boolean isenable) {
        super.enableEdition(isenable);
        centerparticlebt.setEnabled(isenable);
        if (ppicker.getMode() != Mode.Review) {
            autopickchb.setEnabled(isenable);
            thresholdpn.setEnabled(isenable);
        }
        saveandexitbt.setEnabled(isenable);
         if(ppicker.isScipionSave())
        {
            Color color = isenable? XmippWindowUtil.firebrick: XmippWindowUtil.LIGHT_BLUE; 
            Color forecolor = isenable? Color.WHITE: Color.GRAY;
            saveandexitbt.setBackground(color);
            saveandexitbt.setForeground(forecolor);
        }
       
    }

    public void initMenuBar() {
        mb = new JMenuBar();

		// Setting menus
        exportmi = new JMenuItem("Export coordinates...", XmippResource.getIcon("export_wiz.gif"));

        exportmi.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                XmippFileChooser fc = new XmippFileChooser();
                int returnVal = fc.showOpenDialog(SupervisedPickerJFrame.this);

                try {
                    if (returnVal == XmippFileChooser.APPROVE_OPTION) {
                        File file = fc.getSelectedFile();
                        ppicker.exportParticles(file.getAbsolutePath());
                        showMessage("Export successful");
                    }
                } catch (Exception ex) {
                    showException(ex);
                }
            }
        });
        filemn.add(importmi);
        if (ppicker.getMode() != Mode.Manual) {
            importmi.setEnabled(false);
        }
        filemn.add(exportmi);
        filemn.add(exitmi);

        JMenu windowmn = new JMenu(bundle.getString("window"));

        mb.add(filemn);
        mb.add(filtersmn);
        mb.add(windowmn);
        mb.add(helpmn);
        // importffilemi.setText("Import from File...");

        windowmn.add(pmi);
        windowmn.add(ijmi);

       
        templatesmi = new JMenuItem("Templates");
        templatesmi.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent arg0) {
                loadTemplates();
            }
        });
        windowmn.add(templatesmi);

    }

   

    private void initThresholdPane() {

        // Create the panel for the threshold
        thresholdpn = new JPanel();

        // Add the threshold label
        thresholdlb = new JLabel("Threshold:");
        thresholdpn.add(thresholdlb);

        final ParticlePicker picker = this.getParticlePicker();
        thresholdlb.addMouseListener(new MouseAdapter()
        {
            public void mouseClicked(MouseEvent e)
            {
                // Show deleted particles
                if (e.getClickCount() == 3) {
                    picker.toggleShowDeleted();
                }

            }
        });


        // Add and configure the threshold slider
        thresholdsl = new JSlider(0, 100, 0);
        thresholdsl.setPaintTicks(false);
        thresholdsl.setMajorTickSpacing(5);
        int height = (int) thresholdsl.getPreferredSize().getHeight();
        thresholdsl.setPreferredSize(new Dimension(50, height));
        thresholdpn.add(thresholdsl);

        // Add the threshold text field
        thresholdtf = new JFormattedTextField(NumberFormat.getNumberInstance());
        thresholdtf.setColumns(3);
        thresholdtf.setValue(0);
        thresholdpn.add(thresholdtf);
        thresholdtf.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {

                double threshold = Double.parseDouble(thresholdtf.getText()) * 100;
                setThresholdValue(threshold);

            }
        });

        // Link the slider to the text field
        thresholdsl.addChangeListener(new ChangeListener() {

            @Override
            public void stateChanged(ChangeEvent e) {
                if (thresholdsl.getValueIsAdjusting()) {
                    return;
                }
                double threshold = (double) thresholdsl.getValue() / 100;
                if (getMicrograph().getThreshold() != threshold) {
                    thresholdtf.setText(String.format("%.2f", threshold));
                    setThresholdChanges();
                }
            }
        });
    }

    private void setThresholdValue(double threshold) {
        if (Math.abs(threshold) <= 100) {
            thresholdsl.setValue((int) threshold);
            setThresholdChanges();
        }
    }

    private void setThresholdChanges() {

        getMicrograph().setThreshold(getThreshold());
        updateMicrographsModel();
        canvas.repaint();
        if (particlesdialog != null) {
            loadParticles(false);
        }

        this.setThresholdChanged(true);
    }

    private void initMicrographsPane() {
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.insets = new Insets(0, 5, 0, 5);
        constraints.anchor = GridBagConstraints.NORTHWEST;
        micrographpn = new JPanel(new GridBagLayout());
        micrographpn.setBorder(BorderFactory.createTitledBorder("Micrographs"));
        
        
        if(ppicker.containsPSD())
        {
            iconbt = new JButton(Micrograph.getNoImageIcon());
            iconbt.setToolTipText("Load CTF Profile");
            iconbt.setBorderPainted(false);
            iconbt.setContentAreaFilled(false);
            iconbt.setFocusPainted(false);
            iconbt.setOpaque(false);
            iconbt.setMargin(new Insets(0, 0, 0, 0));
            iconbt.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent arg0) {
                    String psd = getMicrograph().getPSD();
                    String ctf = getMicrograph().getCTF();
                    if (psd != null && ctf != null) {
                        try {
                            new CTFAnalyzerJFrame(psd, ctf, psd);
                        } catch (Exception ex) {
                            Logger.getLogger(SupervisedPickerJFrame.class.getName()).log(Level.SEVERE, null, ex);
                        }
                    }

                }
            });
            micrographpn.add(iconbt, XmippWindowUtil.getConstraints(constraints, 1, 0, 1));
        }
        
        JScrollPane sp = new JScrollPane();
        micrographsmd = new MicrographsTableModel(this);
        micrographstb.setModel(micrographsmd);
        formatMicrographsTable();
        sp.setViewportView(micrographstb);
        micrographpn.add(sp, XmippWindowUtil.getConstraints(constraints, 0, 0, 1));

        // Info panel
        JPanel infopn = new JPanel();
        manuallb = new JLabel(Integer.toString(ppicker.getManualParticlesNumber()));
        autolb = new JLabel(Integer.toString(ppicker.getAutomaticParticlesNumber()));
        infopn.add(new JLabel("Manual:"));
        infopn.add(manuallb);
        infopn.add(new JLabel("Automatic:"));
        infopn.add(autolb);

        // Saved
        savedlb = new JLabel("");
        infopn.add(savedlb);

        micrographpn.add(infopn, XmippWindowUtil.getConstraints(constraints, 0, 1, 1));
        JPanel buttonspn = new JPanel(new FlowLayout(FlowLayout.LEFT));

        buttonspn.add(resetbt);
        buttonspn.add(usezoombt);
        micrographpn.add(buttonspn, XmippWindowUtil.getConstraints(constraints, 0, 2, 2));

    }

    protected synchronized void loadMicrograph() {

        if (micrographstb.getSelectedRow() == -1) {
            return;// Probably from fireTableDataChanged raised
        }		// is same micrograph??
        int micindex = ppicker.getMicrographIndex();
        
        int index = micrographstb.getSelectedRow();
        if (micindex == index && canvas != null && canvas.getIw().isVisible()) {
            return;
        }

        // Save particles first before moving and loading the clicked micrograph.
        if (ppicker.isChanged() && ppicker.getMode() != Mode.Supervised)
        {
            if (!saveData(false)) {
                micrographstb.getSelectionModel().setSelectionInterval(micindex,micindex);
                return;
            }
        }

        SupervisedPickerMicrograph next = ppicker.getMicrographs().get(index);
        SupervisedPickerMicrograph current = getMicrograph();
        current.resetParticlesRectangle();
        if (!current.equals(next))// app just started
        {
            int result = tryCorrectAndAutopick(current, next);

            if (result == XmippQuestionDialog.CANCEL_OPTION) {
                micrographstb.setRowSelectionInterval(micindex, micindex);
                return;
            }
            
        }
        ppicker.setMicrograph(next);
        ppicker.saveConfig();
        Mode mode = ppicker.getMode();

        if (mode == Mode.Supervised) {
            resetbt.setEnabled(next.getState() != MicrographState.Manual);
            double threshold = next.getThreshold();
            if(threshold == 0 && next.getAutomaticParticlesNumber() == 0)
            {
                threshold = current.getThreshold();
                next.setThreshold(threshold);
                this.setThresholdChanged(true);
            }else{
                this.setThresholdChanged(false);
            }
            thresholdsl.setValue((int) (threshold * 100));
            thresholdtf.setValue(threshold);
        }
        else if (mode == Mode.Automatic) {
            resetbt.setEnabled(false);
        }

        initializeCanvas();
        if(ppicker.containsPSD())
            iconbt.setIcon(next.getCTFIcon());

        pack();

        if (particlesdialog != null) {
            loadParticles(false);
        }

    }

    private int tryCorrectAndAutopick(SupervisedPickerMicrograph current, SupervisedPickerMicrograph next) {
        int result = 3;

        boolean isautopick = (ppicker.getMode() == Mode.Supervised || ppicker.getMode() == Mode.Automatic) && next.getState() == MicrographState.Available;
        if (ppicker.isCorrectPending()) {
            // Do not ask...always correct ans autopick.
            // String msg = String.format("Would you like to correct training with added and deleted particles from micrograph %s?", current.getName());
            // result = XmippDialog.showQuestionYesNoCancel(this, msg);
            result = XmippQuestionDialog.YES_OPTION;
            //if (result == XmippQuestionDialog.YES_OPTION) {
            ppicker.correctAndAutopick(this, current, next);
            isautopick = false;
            //}
            //if (result == XmippQuestionDialog.CANCEL_OPTION)
            //    isautopick = false;
        }
        if (isautopick)// if not done before
        {
            ppicker.autopick(this, next);
        }
        
        return result;
    }

    protected void resetMicrograph() {
        ppicker.resetMicrograph(getMicrograph());
        canvas.refreshActive(null);
        updateMicrographsModel();
        ppicker.updateTemplatesStack(false);
        
        if (ppicker.getMode() == Mode.Supervised) {
            ppicker.autopick(this, getMicrograph());
        }
    }

    public boolean isPickingAvailable(MouseEvent e) {
        if (!super.isPickingAvailable(e)) {
            return false;
        }
        return getMicrograph().getState() != MicrographState.Corrected;
    }

    public SupervisedPickerMicrograph getTrainMic() {
        String trainmsg = "Classifier training for autopick requires that the previous micrographs and the particle's region detected to be fully picked. "
                + "Are you sure you want to continue?";
        if (ppicker.getManualParticlesNumber() < ppicker.getParticlesThreshold()) {
            XmippDialog.showInfo(this, String
                    .format("You should have at least %s particles to go to %s mode", ppicker.getParticlesThreshold(), Mode.Supervised));
            return null;
        }
        boolean isvalidrect = false;
        int index = micrographstb.getSelectedRow();
        SupervisedPickerMicrograph rectmic = null;
        while (index >= 0) 
        {
                rectmic = ppicker.getMicrographs().get(index);
                if (rectmic.hasManualParticles()) 
                {
                    rectmic.initParticlesRectangle(ppicker);
                    isvalidrect = rectmic.isValid(ppicker);
                    if(isvalidrect)
                        break;
                    else
                        rectmic.resetParticlesRectangle();
                }
            index --;
        } 
        if(index < 0)
        {
            index = micrographstb.getSelectedRow() + 1;
            while (index < ppicker.getMicrographs().size()) 
            {
                    rectmic = ppicker.getMicrographs().get(index);
                    if (rectmic.hasManualParticles()) 
                    {
                        rectmic.initParticlesRectangle(ppicker);
                        isvalidrect = rectmic.isValid(ppicker);
                        if(isvalidrect)
                            break;
                    }
                index ++;
            }
        }
        
        if(index < 0 || index == ppicker.getMicrographs().size())
        {
            canvas.repaint();
            XmippDialog.showError(this, "No valid training rectangle could be found in micrographs picked. Particles might be too few.");
            rectmic.resetParticlesRectangle();
            canvas.repaint();
            return null;
        }
        trainmsg = "";
        if(rectmic.equals(getMicrograph()))
        {
            canvas.repaint();
            trainmsg += "Classifier requires that the region detected and the micrographs previously picked to be fully picked. "
                    + "Are you sure you want to continue?";
        }
        else
        	trainmsg += "Classifier requires that the micrographs previously picked to be fully picked. "
                            + "Are you sure you want to continue?";
        int result = XmippDialog
                .showQuestionYesNoCancel(this, trainmsg);
        if (result == XmippQuestionDialog.CANCEL_OPTION) {
            canvas.repaint();
            rectmic.resetParticlesRectangle();
        }
        if(result != XmippQuestionDialog.YES_OPTION)
        	rectmic = null;
        return rectmic;
    }

    

    public void close() {
        if (ppicker.isCorrectPending()) {
            boolean iscorrect = XmippDialog.showQuestion(this, "Would you like to correct training with added and deleted particles?");
            if (iscorrect) {
                ppicker.correct();
            }
        }
        super.close();
    }

    public String getResetMsg() {
        String msg = super.getResetMsg();
        if (ppicker.getMode() == Mode.Supervised) {
            msg += "\nParticles will be automatically picked again";
        }
        return msg;
    }
    
    
    
    protected void initGenericPickerPane()
    {
    	gpickerpn = new JPanel(new FlowLayout(FlowLayout.LEFT));
    	gpickerpn.setBorder(BorderFactory.createTitledBorder("Autopick"));
    	List<Classifier.Parameter> parameters = ppicker.getClassifier().getParams();
    	paramtfs = new HashMap<JTextField, Classifier.Parameter>();
    	JTextField tf;
    	JLabel lb;
    	for(Classifier.Parameter param: parameters)
    	{
    		lb = new JLabel(param.label);
    		lb.setToolTipText(param.help);
    		gpickerpn.add(lb);
    		tf = new JTextField(3);
    		tf.setToolTipText(param.help);
    		tf.setText(param.value);
    		tf.setActionCommand(param.name);
    		tf.addFocusListener(new FocusListener()
			{
				
				@Override
				public void focusLost(FocusEvent e)
				{
					if(e.isTemporary())
						return;
					JTextField tf = (JTextField)e.getComponent();
					updateParam(tf);
				}
				
				@Override
				public void focusGained(FocusEvent e)
				{
					// TODO Auto-generated method stub
					
				}
			});
    		tf.addActionListener(new ActionListener()
			{
				@Override
				public void actionPerformed(ActionEvent e)
				{
					JTextField tf = (JTextField)e.getSource();
					updateParam(tf);
					for(SupervisedPickerMicrograph m: ppicker.getMicrographs())
						ppicker.resetMicrograph(m);
					ppicker.autopick(SupervisedPickerJFrame.this, getMicrograph());

				}
			});
    		gpickerpn.add(tf);
    		paramtfs.put(tf, param);
    		if(param.name.equals("diameter") || param.name.equals("boxSize"))//classifier diameter and particle size are the same
    			updateSize(Integer.parseInt(tf.getText()));
    	}
    	autopickbt = XmippWindowUtil.getTextButton("Run", new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				for(SupervisedPickerMicrograph m: ppicker.getMicrographs())
					ppicker.resetMicrograph(m);
				ppicker.autopick(SupervisedPickerJFrame.this, getMicrograph());
				
			}});
    	autopickbt.setBackground(XmippWindowUtil.firebrick);
    	autopickbt.setForeground(Color.WHITE);
    	gpickerpn.add(autopickbt);
    }
    
    protected void updateParam(JTextField tf)
    {
		Classifier.Parameter param = paramtfs.get(tf);
		if(param.value.equals(tf.getText()))
			return;
		if(param.name.equals("diameter") || param.name.equals("boxSize"))//classifier diameter and particle size are the same
			updateSize(Integer.parseInt(tf.getText()));
		param.value = tf.getText();
    }
    @Override
    protected void showHideModeParameters(){
        super.showHideModeParameters();
        centerparticlebt.setVisible(isNormalMode());
    }

}
